==> gardener-cicd-libs-1.2051.0.tar.gz: gardener-cicd-libs-1.2051.0/delivery/util.py <==
import logging

import awesomeversion

import delivery.client
import delivery.model
import delivery.util
import unixutil.model as um


logger = logging.getLogger(__name__)


def find_branch_info(
    os_id: um.OperatingSystemId,
    os_infos: list[delivery.model.OsReleaseInfo],
) -> delivery.model.OsReleaseInfo:
    if not os_id.ID:
        return None # os-id could not be determined

    os_version = os_id.VERSION_ID

    def version_candidates():
        yield os_version
        yield f'v{os_version}'

        parts = os_version.split('.')

        if len(parts) == 1:
            return

        yield parts[0]
        yield 'v' + parts[0]

        yield '.'.join(parts[:2]) # strip parts after minor
        yield 'v' + '.'.join(parts[:2]) # strip parts after minor

    candidates = tuple(version_candidates())

    for os_info in os_infos:
        for candidate in candidates:
            if os_info.name == candidate:
                return os_info

    logger.warning(f'did not find branch-info for {os_id=}')


def branch_reached_eol(
    os_id: um.OperatingSystemId,
    os_infos: list[delivery.model.OsReleaseInfo],
) -> bool:
    branch_info = find_branch_info(
        os_id=os_id,
        os_infos=os_infos,
    )
    if not branch_info:
        return False

    return branch_info.reached_eol()


def update_available(
    os_id: um.OperatingSystemId,
    os_infos: list[delivery.model.OsReleaseInfo],
) -> bool:
    branch_info = find_branch_info(
        os_id=os_id,
        os_infos=os_infos,
    )
    if not branch_info:
        return False

    if not branch_info.greatest_version:
        logger.warning(f'no greatest version known for {os_id.NAME=} {os_id.VERSION_ID=}')
        return False

    return branch_info.greatest_version > awesomeversion.AwesomeVersion(os_id.VERSION_ID)

==> gardener-cicd-libs-1.2149.0.tar.gz: gardener-cicd-libs-1.2149.0/delivery/util.py <==
import logging

import awesomeversion

import delivery.client
import delivery.model
import delivery.util
import unixutil.model as um


logger = logging.getLogger(__name__)


def find_branch_info(
    os_id: um.OperatingSystemId,
    os_infos: list[delivery.model.OsReleaseInfo],
) -> delivery.model.OsReleaseInfo:
    if not os_id.ID:
        return None # os-id could not be determined

    os_version = os_id.VERSION_ID

    def version_candidates():
        yield os_version
        yield f'v{os_version}'

        parts = os_version.split('.')

        if len(parts) == 1:
            return

        yield parts[0]
        yield 'v' + parts[0]

        yield '.'.join(parts[:2]) # strip parts after minor
        yield 'v' + '.'.join(parts[:2]) # strip parts after minor

    candidates = tuple(version_candidates())

    for os_info in os_infos:
        for candidate in candidates:
            if os_info.name == candidate:
                return os_info

    logger.warning(f'did not find branch-info for {os_id=}')


def branch_reached_eol(
    os_id: um.OperatingSystemId,
    os_infos: list[delivery.model.OsReleaseInfo],
) -> bool:
    branch_info = find_branch_info(
        os_id=os_id,
        os_infos=os_infos,
    )
    if not branch_info:
        return False

    return branch_info.reached_eol


def update_available(
    os_id: um.OperatingSystemId,
    os_infos: list[delivery.model.OsReleaseInfo],
    ignore_if_patchlevel_is_next_to_greatest=False,
) -> bool:
    branch_info = find_branch_info(
        os_id=os_id,
        os_infos=os_infos,
    )
    if not branch_info:
        return False

    if not branch_info.greatest_version:
        return False

    version = awesomeversion.AwesomeVersion(os_id.VERSION_ID)

    greater_version_available = (gv := branch_info.greatest_version) > version

    if not greater_version_available or not ignore_if_patchlevel_is_next_to_greatest:
        return greater_version_available

    # there is greated version; check whether difference is not more than one patchlevel
    # check whether both versions actually _have_ patchlevel
    if not gv.patch or not version.patch:
        return greater_version_available

    # assume "next-to-greatest" patch-level to be "great enogh"
    patch_diff = int(gv.patch) - int(version.patch)
    return patch_diff < 2
